package fr.inra.pappso.libodsstream;

/*******************************************************************************
 * Copyright (c) 2011 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 * 
 * This file is part of libOdsStream.
 * 
 *     libOdsStream is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     libOdsStream is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with libOdsStream.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr> - initial API and implementation
 ******************************************************************************/

import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URI;
import java.sql.Timestamp;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.zip.CRC32;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.Duration;
import javax.xml.stream.XMLStreamException;

import fr.inra.pappso.libodsstream.structure.ContentXml;
import fr.inra.pappso.libodsstream.structure.ManifestXml;
import fr.inra.pappso.libodsstream.structure.MetaXml;
import fr.inra.pappso.libodsstream.structure.StylesXml;

/**
 * Write an ODS document to either a file or an OutputStream
 * 
 * @author Olivier Langella <Olivier.Langella@moulon.inra.fr>
 * 
 */
public class OdsDocWriter {

	private ZipOutputStream zipOutputStream;
	private ContentXml content;
	private boolean isClosed = false;

	/**
	 * initialize an ODS writer with a filename
	 * 
	 * This will create an ODS file named outputFilename on you file system.
	 * 
	 * @param outputFilename
	 * @throws IOException
	 * @throws XMLStreamException
	 * @throws DatatypeConfigurationException
	 */
	public OdsDocWriter(String outputFilename) throws IOException,
			XMLStreamException, DatatypeConfigurationException {
		FileOutputStream dest = new FileOutputStream(outputFilename);
		zipOutputStream = new ZipOutputStream(new BufferedOutputStream(dest));
		InitStructure();
	}

	/**
	 * initialize an ODS writer with an OutputStream
	 * 
	 * This will create an ODS output stream. You can use this to create ODS
	 * output on the fly (usefull in a webapp for example).
	 * 
	 * @param out
	 * @throws IOException
	 * @throws XMLStreamException
	 * @throws DatatypeConfigurationException
	 */
	public OdsDocWriter(OutputStream out) throws IOException,
			XMLStreamException, DatatypeConfigurationException {
		zipOutputStream = new ZipOutputStream(new BufferedOutputStream(out));
		InitStructure();
	}

	private void InitStructure() throws IOException, XMLStreamException,
			DatatypeConfigurationException {

		CRC32 tempCRC = new CRC32();
		byte[] bytes = new String(
				"application/vnd.oasis.opendocument.spreadsheet").getBytes();
		tempCRC.update(bytes, 0, bytes.length);
		ZipEntry entry = new ZipEntry("mimetype");
		entry.setMethod(ZipEntry.STORED);
		entry.setSize((long) bytes.length);
		entry.setCrc(tempCRC.getValue());
		zipOutputStream.putNextEntry(entry);
		zipOutputStream.write(bytes, 0, bytes.length);

		new ManifestXml(this.zipOutputStream);
		// meta.xml
		new MetaXml(this.zipOutputStream);
		// mimetype
		// application/vnd.oasis.opendocument.spreadsheet
		// settings.xml
		// styles.xml

		// content.xml

		this.content = new ContentXml(this.zipOutputStream);
	}

	/**
	 * ends the document creation. It is required to obtain a correct output
	 * 
	 * @throws Exception
	 */
	synchronized public void close() throws Exception {
		if (isClosed == true) {
			// already closed
		} else {
			isClosed = true;
			content.close();
			new StylesXml(this.zipOutputStream);
			zipOutputStream.close();
			zipOutputStream = null;
		}
	}

	/**
	 * adds a new datasheet to the document
	 * 
	 * @param sheetName
	 * @throws Exception
	 */
	public void writeSheet(String sheetName) throws Exception {
		this.checkClosed();
		this.content.writeSheet(sheetName);
	}

	synchronized private void checkClosed() throws Exception {
		if (this.isClosed) {
			String message = "OdsDocWriter is closed";
			throw new Exception(message);
		}
	}

	/**
	 * adds a new line to the document
	 * 
	 * @throws Exception
	 */
	public void writeLine() throws Exception {
		this.checkClosed();
		this.content.writeLine();
	}

	/**
	 * adds a new text cell to the document
	 * 
	 * @param text
	 * @throws Exception
	 */
	public void writeCell(String text) throws Exception {
		this.checkClosed();
		if (text == null) {
			this.writeEmptyCell();
			return;
		}
		this.content.writeCell(text);
	}

	/**
	 * writes an empty cell
	 * 
	 * @throws Exception
	 */
	public void writeEmptyCell() throws Exception {
		this.checkClosed();
		this.content.writeCell("");
	}

	/**
	 * adds a new cell containing an integer
	 * 
	 * @param value
	 * @throws Exception
	 */
	public void writeCell(int value) throws Exception {
		this.checkClosed();
		this.content.writeCell(value);
	}

	/**
	 * adds a new cell containing a float
	 * 
	 * @param value
	 * @throws Exception
	 */
	public void writeCell(float value) throws Exception {
		this.checkClosed();
		this.content.writeCell(value);
	}

	/**
	 * adds a new cell containing a double
	 * 
	 * @param value
	 * @throws Exception
	 */
	public void writeCell(double value) throws Exception {
		this.checkClosed();
		this.content.writeCell(value);
	}

	/**
	 * adds a new cell containing a boolean
	 * 
	 * @param value
	 * @throws Exception
	 */
	public void writeCell(boolean value) throws Exception {
		this.checkClosed();
		this.content.writeCell(value);
	}

	/**
	 * adds a new cell containing a date
	 * 
	 * @param date
	 * @throws Exception
	 */
	public void writeCell(Date date) throws Exception {
		if (date == null) {
			this.writeEmptyCell();
			return;
		}
		this.checkClosed();
		this.content.writeCell(date);
	}

	/**
	 * adds a new cell containing a timestamp
	 * 
	 * @param date
	 * @throws Exception
	 */
	public void writeCell(Timestamp date) throws Exception {
		if (date == null) {
			this.writeEmptyCell();
			return;
		}
		this.checkClosed();
		this.content.writeCell(new Date(date.getTime()));
	}

	/**
	 * adds a new cell containing a date
	 * 
	 * @param date
	 * @throws Exception
	 */
	public void writeCell(GregorianCalendar date) throws Exception {
		if (date == null) {
			this.writeEmptyCell();
			return;
		}
		this.checkClosed();
		this.content.writeCell(date);
	}

	/**
	 * NOT WORKING, adds a new cell containing a duration
	 * 
	 * @param duration
	 * @throws Exception
	 */
	public void writeCell(Duration duration) throws Exception {
		if (duration == null) {
			this.writeEmptyCell();
			return;
		}
		this.checkClosed();
		this.content.writeCell(duration);
	}

	/**
	 * adds a new cell containing a text inside an href link
	 * 
	 * @param theUri
	 * @param text
	 * @throws Exception
	 */
	public void writeCell(URI theUri, String text) throws Exception {
		this.checkClosed();
		this.content.writeCell(theUri, text);
	}

	/**
	 * adds a new cell containing a java object
	 * 
	 * @param object
	 * @throws Exception
	 */
	public void writeCell(Object object) throws Exception {
		if (object == null) {
			this.writeEmptyCell();
			return;
		}
		if (object.getClass().equals(String.class)) {
			this.content.writeCell((String) object);
		} else if ((object.getClass().equals(Float.class))
				|| (object.getClass().equals(Double.class))) {
			this.content.writeCell((Double) object);
		} else if (object.getClass().equals(Integer.class)) {
			this.content.writeCell((Integer) object);
		} else if (object.getClass().equals(Long.class)) {
			this.content.writeCell((Long) object);
		} else if (object.getClass().equals(Date.class)) {
			this.content.writeCell((Date) object);
		} else if (object.getClass().equals(Timestamp.class)) {
			this.content.writeCell((Timestamp) object);
		} else {
			throw new Exception("Class '" + object.getClass().getName()
					+ "' not allowed in libodsstream");
		}
	}

}
