/*******************************************************************************
 * Copyright (c) 2011 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 * 
 * This file is part of libOdsStream.
 * 
 *     libOdsStream is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     libOdsStream is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with libOdsStream.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr> - initial API and implementation
 ******************************************************************************/

package fr.inra.pappso.libodsstream.saxreader;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.XMLReaderFactory;

import fr.inra.pappso.libodsstream.reader.OdsCell;
import fr.inra.pappso.libodsstream.reader.OdsSheet;
import fr.inra.pappso.libodsstream.saxreader.MyErrorHandler;
import fr.inra.pappso.libodsstream.saxreader.SaxHandlerContentXml;

/**
 * Reads an ODS document from either a file or an input stream. To use it,
 * create first a new class extending OdsDocReader and override needed functions
 * 
 * @author Olivier Langella <Olivier.Langella@moulon.inra.fr>
 * 
 */
abstract public class SaxOdsDocReader {
	private InputStream input;

	private long lineNumber = 0;
	private long sheetNumber = 0;
	private String sheetName = null;
	private long columnNumber = 0;

	static char[] alphabet = new char[26];

	static {
		int i = 0;
		for (char c = 'A'; c <= 'Z'; ++c) {
			// System.out.println(" i " + i + " c " + c);
			alphabet[i] = c;
			i++;
		}
	}

	/**
	 * creates a ODS file reader
	 * 
	 * @param inputFilename
	 * @return
	 * @throws IOException
	 */
	public SaxOdsDocReader(String inputFilename) throws IOException {

		input = new FileInputStream(inputFilename);
	}

	/**
	 * creates an ODS reader from an input stream
	 * 
	 * @param in
	 * @throws IOException
	 */
	public SaxOdsDocReader(InputStream in) throws IOException {
		input = in;
	}

	/**
	 * trigger the ODS document parsing : required in your code
	 * 
	 * @throws Exception
	 */

	final public void readOds() throws Exception {
		if (input == null) {
			throw new java.io.IOException(
					"input is closed (perhaps already read)");
		}

		ZipInputStream zipInputStream = new ZipInputStream(
				new BufferedInputStream(input));
		ZipEntry entry = null;
		boolean readOk = false;
		while (readOk == false) {

			try {
				entry = zipInputStream.getNextEntry();
			} catch (java.io.IOException e) {
				// Exception in thread "main" java.io.IOException: Stream Closed
				throw e;
			}
			if (entry == null) {
				readOk = true;
				break;
			}
			// System.out.println("Extracting: " + entry);
			if (entry.getName().equals("content.xml")) {
				SaxHandlerContentXml handler;
				try {
					handler = new SaxHandlerContentXml(this);
				} catch (Exception e) {
					String message = "ERROR reading content.xml in this ods file : "
							+ e.getMessage();
					// logger.debug("Exception e " + message);
					throw new IOException(message);
				}
				parseInputStreamContentXml(handler, zipInputStream);
				readOk = true;
				break;
			}
		}
		this.endDocument();

		input = null;
	}

	final private void parseInputStreamContentXml(SaxHandlerContentXml handler,
			InputStream is) throws IOException {
		try {

			org.xml.sax.XMLReader saxParser;
			try {
				saxParser = XMLReaderFactory.createXMLReader();
			} catch (SAXException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new Exception("ERROR creating XMLReaderFactory "
						+ e.getMessage());
			}
			saxParser.setContentHandler(handler);
			saxParser.setErrorHandler(new MyErrorHandler());
			saxParser.parse(new InputSource(is));

		} catch (Exception e) {
			String message = "ERROR reading content.xml in this ods file : "
					+ e.getMessage();
			// logger.debug("Exception e " + message);
			throw new IOException(message);
		}
	}

	/**
	 * callback that indicates the begining of a data sheet. Override it in
	 * order to retrieve information about the current data sheet.
	 * 
	 * @param sheet
	 * @throws Exception
	 */
	public void startSheet(OdsSheet sheet) throws Exception {
	}

	/**
	 * callback that indicates the end of the current data sheet. Override it if
	 * needed
	 * 
	 * @throws Exception
	 */
	public void endSheet() throws Exception {
	}

	/**
	 * callback that indicates a new line start. Override it if needed.
	 * 
	 * @throws Exception
	 */
	public void startLine() throws Exception {
	}

	/**
	 * callback that indicates a line ending. Override it if needed.
	 * 
	 * @throws Exception
	 */
	public void endLine() throws Exception {
	}

	/**
	 * callback that report the content of the current cell in a dedicated Cell
	 * object. Override it if you need to retrieve cell content.
	 * 
	 * @throws Exception
	 */
	public void setCell(OdsCell cell) throws Exception {
	}

	/**
	 * callback that report the end of the ODS documen. Override it if you need
	 * to know that reading is finished.
	 * 
	 * @throws Exception
	 */
	public void endDocument() throws Exception {
	}

	protected void setSheetName(String name) {
		this.sheetName = name;
		this.sheetNumber++;
		this.lineNumber = 0;
	}

	protected void incLineNumber() {
		this.lineNumber++;
		this.columnNumber = 0;
	}

	public void incColumnNumber() {
		this.columnNumber++;
	}

	/**
	 * get current sheet name
	 * 
	 */
	final public String getSheetPositionStr() {
		return this.sheetName;
	}

	/**
	 * get current sheet number
	 * 
	 */
	final public long getSheetPosition() {
		return this.sheetNumber;
	}

	/**
	 * get current line number
	 * 
	 */
	final public long getLinePosition() {
		return this.lineNumber;
	}

	/**
	 * get current column number
	 * 
	 */
	final public long getColumnPosition() {
		return this.columnNumber;
	}

	/**
	 * get current column name
	 * 
	 */
	final public String getColumnPositionStr() {
		String positionStr = "" + this.getColumnPosition();

		if (this.getColumnPosition() < 27) {
			positionStr = "" + alphabet[(int) (this.getColumnPosition() - 1)];
		}

		// A -> Z
		return positionStr;
	}
}
