/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 * 
 * This file is part of XTandemPipeline.
 * 
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors:
 *     Benoit Valot <Benoit.Valot@moulon.inra.fr>
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline.output.SpreadSheets;

import java.io.IOException;
import java.io.OutputStream;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.stream.XMLStreamException;

import org.apache.log4j.Logger;

import fr.inra.pappso.libodsstream.OdsDocWriter;
import fr.inra.pappso.xtandempipeline.Utils;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetPhospho;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetProt;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;

public class OdsPhosphoProteinOutput {
	private static final Logger logger = Logger
			.getLogger(OdsPhosphoProteinOutput.class);

	protected OdsDocWriter odsTable;

	protected OdsPhosphoProteinOutput() {

	}

	public OdsPhosphoProteinOutput(OutputStream outputStream)
			throws MSMSException {
		try {
			odsTable = new OdsDocWriter(outputStream);
			odsTable.writeSheet("proteins");
		} catch (IOException e) {
			String message = "IOException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (DatatypeConfigurationException e) {
			String message = "DatatypeConfigurationException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public OdsPhosphoProteinOutput(OdsDocWriter odsTable) throws MSMSException {
		try {
			this.odsTable = odsTable;
			odsTable.writeSheet("proteins");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public void close() throws MSMSException {
		try {
			odsTable.close();
		} catch (Exception e) {
			String message = "Exception closing OdsPhosphoProteinOutput "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void writeHeaders(Identification ident) throws MSMSException {
		try {
			// Group ID PhosphoIsland ID Phosphosite ID Description MW Positions
			// Spectra Uniques Redundancy

			odsTable.writeLine();
			odsTable.writeCell("Group ID");
			odsTable.writeCell("PhosphoIsland ID");
			odsTable.writeCell("Phosphosite ID");
			odsTable.writeCell("Description");
			odsTable.writeCell("MW");
			odsTable.writeCell("Positions");
			odsTable.writeCell("Spectra");
			odsTable.writeCell("Uniques");
			odsTable.writeCell("number of proteins in this sharing this phosphosite");

		} catch (XMLStreamException e) {
			String message = "XMLStreamException in writeHeaders() "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in writeHeaders() " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}

	}

	public void write(Identification ident) throws MSMSException {
		try {

			this.writeHeaders(ident);

			for (Group group : ident.getGrouping().getGroupList()) {
				for (SubGroup sg : group.getSubGroupSet().getSubGroupList()) {
					// proteins
					int k = 0;
					for (HashSampleScanSetProt hashProt : sg
							.getHashSampleScanSetProtList()) {
						k++;
						this.writeOnePhosphoProtein(hashProt, group, sg, k);
					}
				}
			}
			odsTable.writeLine();
			odsTable.writeLine();

		} catch (XMLStreamException e) {
			String message = "XMLStreamException in write(Identification ident) "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in write(Identification ident) "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void writeOnePhosphoProtein(HashSampleScanSetProt hashProt,
			Group group, SubGroup sg, int k) throws MSMSException {
		try {
			logger.debug("writeOnePhosphoProtein begin " + group + sg);
			odsTable.writeLine();

			Match top = (Match) hashProt.getGrpProtMatch();
			HashSampleScanSetPhospho phospho = (HashSampleScanSetPhospho) hashProt;

			// odsTable.writeCell("Group ID");
			odsTable.writeCell(Utils.getPappsoGroupId(group));

			// odsTable.writeCell("PhosphoIsland ID");
			odsTable.writeCell(Utils.getPappsoSubGroupId(group, sg));

			// odsTable.writeCell("Phosphosite ID");
			odsTable.writeCell(Utils.getPappsoGroupingNumber(group, sg, k));

			// odsTable.writeCell("Description");
			odsTable.writeCell(top.get_protein_match().get_description());

			// odsTable.writeCell("MW");
			odsTable.writeCell(top.get_protein_match().get_mw());

			// odsTable.writeCell("Positions");
			odsTable.writeCell(phospho.getPhosphoPositionToPrint().trim());

			// odsTable.writeCell("Spectra");
			odsTable.writeCell(phospho.size());

			//odsTable.writeCell("Uniques");
			odsTable.writeCell(phospho.getUniquePeptideHashSampleScan());

			//odsTable.writeCell("number of proteins sharing this phosphosite");
			odsTable.writeCell(sg.getHashSampleScanSetProtSet().size());

			// + this.get_subgroup_number(group, sg) + "\t"
			// + top.get_protein_match().get_description());

			// S.append("\t" + top.get_protein_match().get_mw());

			// S.append("\t" + "\"" + phospho.getPhosphoPositionToPrint()
			// + "\"");

			// S.append("\t" + phospho.size());

			// S.append("\t" + phospho.getUniquePeptideHashSampleScan());
			// S.append("\t");
			

			logger.debug("writeOnePhosphoProtein end");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException in writeOnePhosphoProtein "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in writeOnePhosphoProtein "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}
}
