/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 * 
 * This file is part of XTandemPipeline.
 * 
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors:
 *     Benoit Valot <Benoit.Valot@moulon.inra.fr>
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline.output.SpreadSheets;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Collection;
import java.util.HashSet;
import java.util.TreeSet;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.stream.XMLStreamException;

import org.apache.log4j.Logger;

import fr.inra.pappso.libodsstream.OdsDocWriter;
import fr.inra.pappso.xtandempipeline.Utils;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.MsRun;
import fr.inra.pappso.xtandempipeline.class_msms.Protein;
import fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein.PeptideMass;
import fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein.PeptideMassSet;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetProt;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;
import fr.inra.pappso.xtandempipeline.grouping.SubGroupSet;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpPeptide;

public class OdsSpectraOutput {
	private static final Logger logger = Logger
			.getLogger(OdsSpectraOutput.class);

	protected OdsDocWriter odsTable;
	private PeptideMassSet pepMassSet;

	protected OdsSpectraOutput() {

	}

	public OdsSpectraOutput(OutputStream outputStream) throws MSMSException {
		try {
			odsTable = new OdsDocWriter(outputStream);
			odsTable.writeSheet("spectra");
		} catch (IOException e) {
			String message = "IOException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (DatatypeConfigurationException e) {
			String message = "DatatypeConfigurationException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public OdsSpectraOutput(OdsDocWriter odsTable) throws MSMSException {
		try {
			this.odsTable = odsTable;
			odsTable.writeSheet("spectra");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public void close() throws MSMSException {
		try {
			odsTable.close();
		} catch (Exception e) {
			String message = "Exception closing OdsSpectraOutput "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void writeHeaders(Identification ident) throws MSMSException {
		try {
			// Group Peptide ID Sample Scan Rt Sequence (top) Modifs (top)
			// Number of subgroup Sub-groups Ids Best E-value Charge MH+ Obs MH+
			// theo DeltaMH+ Delta-ppm

			odsTable.writeLine();
			odsTable.writeCell("Group ID");
			odsTable.writeCell("Peptide ID");
			odsTable.writeCell("Sample");
			odsTable.writeCell("Scan");
			odsTable.writeCell("Rt");
			odsTable.writeCell("Sequence (top)");
			odsTable.writeCell("Modifs (top)");
			odsTable.writeCell("Number of subgroups");
			odsTable.writeCell("Sub-groups Ids");
			odsTable.writeCell("Best E-value");
			odsTable.writeCell("Charge");
			odsTable.writeCell("MH+ Obs");
			odsTable.writeCell("MH+ theo");
			odsTable.writeCell("DeltaMH+");
			odsTable.writeCell("Delta-ppm");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException in writeHeaders() "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in writeHeaders() " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}

	}

	public void write(Identification ident) throws MSMSException {
		try {

			this.writeHeaders(ident);
			for (Group group : ident.getGrouping().getGroupList()) {
				pepMassSet = new PeptideMassSet();
				int nbpeptide = 0;
				for (SubGroup sg : group.getSubGroupSet().getSubGroupList()) {

					for (HashSampleScanSetProt hashProt : sg
							.getHashSampleScanSetProtList()) {
						for (Peptide peptide : ((Match) hashProt
								.getGrpProtMatch())
								.get_peptide_order_inclus_redon()) {
							// first pass to index each ScanSample and give it a
							// PeptideMass
							nbpeptide++;
							PeptideMass pepMass = pepMassSet
									.addPeptide(peptide);
							pepMass.addSubGroup(sg);
							// HashPeptide p = peptide.getHashPeptide();
							// if
							// (uniquePeptideSet.add(peptide.getHashPeptide()))
							// {
							// this.writeOnePeptide(group, sg, prot, peptide);
							// }

						}
					}
				}
				logger.debug("nb peptide : " + nbpeptide);

				logger.debug("pepMassSet size : " + pepMassSet.size());

				logger.debug("pepMassSet numbering");
				pepMassSet.numbering();

				for (SubGroup sg : group.getSubGroupSet().getSubGroupList()) {
					HashSet<HashSampleScan> uniqueHashSampleScan = new HashSet<HashSampleScan>();

					for (HashSampleScanSetProt hashProt : sg
							.getHashSampleScanSetProtList()) {
						HashSet<PeptideMass> uniquePepMass = new HashSet<PeptideMass>();

						for (Peptide peptide : ((Match) hashProt
								.getGrpProtMatch())
								.get_peptide_order_inclus_redon()) {

							PeptideMass pepMass = pepMassSet
									.getInstance(peptide);
							// pepMass.addSubGroup(sg);
							// HashPeptide p = peptide.getHashPeptide();
							if (uniquePepMass.add(pepMass)) {

								for (HashSampleScan scan : pepMass
										.getHashSampleScanSet()) {

									if (uniqueHashSampleScan.add(scan)) {
										this.writeOneHashSampleScan(ident,
												group, sg, hashProt, pepMass,
												scan);
									}
								}
							}

						}
					}
				}

			}
			odsTable.writeLine();
			odsTable.writeLine();

		} catch (XMLStreamException e) {
			String message = "XMLStreamException in write(Identification ident) "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in write(Identification ident) "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void writeOneHashSampleScan(Identification ident, Group group,
			SubGroup sg, HashSampleScanSetProt hashSampleScanSetProt,
			PeptideMass pepMass, HashSampleScan scan) throws MSMSException {
		try {
			logger.debug("writeOneHashSampleScan begin " + group + pepMass);
			MsRun msRun = (MsRun) scan.getGrpMsSample();
			Protein prot = (Protein) hashSampleScanSetProt.getGrpProtMatch()
					.getGrpProtein();
			Peptide bestPeptide = null;
			Collection<GrpPeptide> peptideList = hashSampleScanSetProt
					.getGrpProtMatch().getGrpPeptides();
			for (GrpPeptide grpPeptide : peptideList) {
				Peptide peptide = (Peptide) grpPeptide;
				if (peptide.getHashSampleScan().equals(scan)) {
					if (pepMass.equals(pepMassSet.getInstance(peptide))) {
						if (bestPeptide == null)
							bestPeptide = peptide;
						if (bestPeptide.get_evalue() > peptide.get_evalue()) {
							bestPeptide = peptide;
						}
					}
				}
			}
			if (bestPeptide == null) {
				//throw new MSMSException("bestPeptide == null");
				return;
			}
			// Peptide bestPeptide = pepMass.getBestEvaluePeptide();

			odsTable.writeLine();
			// odsTable.writeCell("Group ID");
			odsTable.writeCell(Utils.getPappsoGroupId(group));

			// odsTable.writeCell("Peptide ID");
			odsTable.writeCell(Utils.getPappsoPeptideMassId(group, pepMass));

			// odsTable.writeCell("Sample");
			odsTable.writeCell(msRun.getSampleName());

			// odsTable.writeCell("Scan");
			odsTable.writeCell(scan.getScan());

			// odsTable.writeCell("Rt");
			odsTable.writeCell(bestPeptide.get_RT());

			// odsTable.writeCell("Sequence (top)");
			odsTable.writeCell(bestPeptide.get_LI_sequence());

			// odsTable.writeCell("Modifs (top)");
			odsTable.writeCell(bestPeptide.get_Modifs_formater());

			// odsTable.writeCell("Number of subgroups");
			SubGroupSet sgList = group.getSubGroupContaining(scan);
			Collection<String> sgIds = new TreeSet<String>();
			for (SubGroup sgOther : sgList) {
				sgIds.add(Utils.getPappsoSubGroupId(group, sgOther));
			}
			String tagSg = "";
			for (String sgId : sgIds) {
				tagSg += sgId + " ";
			}
			tagSg = tagSg.trim();

			odsTable.writeCell(sgList.size());

			// odsTable.writeCell("Sub-groups Ids");
			odsTable.writeCell(tagSg);

			// odsTable.writeCell("Best E-value");
			odsTable.writeCell(bestPeptide.get_evalue());

			// odsTable.writeCell("Charge");
			odsTable.writeCell(bestPeptide.get_charge());

			// odsTable.writeCell("MH+ Obs");
			odsTable.writeCell(bestPeptide.get_mhplus_obser());

			// odsTable.writeCell("MH+ theo");
			odsTable.writeCell(bestPeptide.get_mhplus_theo());

			// odsTable.writeCell("DeltaMH+");
			odsTable.writeCell(bestPeptide.get_deltamass());

			// odsTable.writeCell("Delta-ppm");
			odsTable.writeCell(((bestPeptide.get_deltamass() / bestPeptide
					.get_mhplus_theo()) * 1000000));

			logger.debug("writeOneHashSampleScan end");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException in writeOneHashSampleScan "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in writeOneHashSampleScan "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}
}
